/*
 * Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
 *
 * This file is part of NRAO Workspaces.
 *
 * Workspaces is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Workspaces is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
 */
import { Component, OnInit, OnChanges, Input, Output, EventEmitter } from '@angular/core';
import { NgbModal } from "@ng-bootstrap/ng-bootstrap";
import { NotificationsService } from "../../services/notifications.service";

/**
 * The SendEmailComponent loads part of an email template in a modal so the user can edit it,
 * add email addresses to the CC header, and send it to the notification service.
 */
@Component({
  selector: 'send-email',
  templateUrl: './send-email.component.html',
  styleUrls: ['./send-email.component.scss']
})
export class SendEmailComponent implements OnInit, OnChanges {
  @Input() modalTitleText: string;
  @Input() customStyle: string = "";
  @Input() shouldDisable: boolean = false;
  @Input() templateName: string = "email";
  @Input() emailParameters: any;
  @Input() allowCC: boolean = true;
  @Input() defaultCC: string = "";
  @Input() customTextTag: string = "custom_text";
  @Output() emailSentEvent = new EventEmitter<string>();
  @Output() emailOpenEvent = new EventEmitter<boolean>();

  private allTemplates: any;
  private isEmailModalOpen: boolean = false;
  private ccAddresses: string;
  private customText: string;
  private defaultText: string;
  private enableCustomText: boolean = true;

  constructor(
    private modalService: NgbModal,
    private notificationsService: NotificationsService) { }

  ngOnInit(): void {
    this.loadEmailTemplate();
    this.ccAddresses = this.defaultCC;
  }

  /**
   * Listens for changes to inputs on this component.
   * @param changes List of input changes
   */
  ngOnChanges(changes): void {
    // Update the defaultCC here to allow observers to change it
    if (changes.defaultCC) {
      this.defaultCC = changes.defaultCC.currentValue;
      this.ccAddresses = this.defaultCC;
    }
  }

  /**
   * Populates the original text of the modal with the content of
   * the email template from the notification service.
   */
  loadEmailTemplate(): void {
    const emailTemplatesObserver = {
      next: (request) => {
        this.allTemplates = request;
        this.allTemplates.forEach(template => {
          if (template.name === this.templateName) {
            // TODO: Make this less janky one day?
            if (template.content.includes(`{{^${this.customTextTag}}}`)) {
              // Load a piece of the template for editing (in between ^customTextTag and /customTextTag)
              this.defaultText = template.content.split(`{{^${this.customTextTag}}}`)[1].split(`{{/${this.customTextTag}}}`)[0];
              this.customText = this.defaultText;
            } else {
              // We can't replace any text in the template if the tag isn't found
              this.defaultText = '';
              this.customText = this.defaultText;
              this.enableCustomText = false;

              console.error("Error when loading custom text to edit, ensure " +
                "'{{#custom_text}}{{custom_text}}{{/custom_text}}{{^custom_text}}DEFAULT TEXT{{/custom_text}}' " +
                "is in the template " + this.templateName);
            }
          }
        });
      },
      error: (error) => console.error("Error when loading email template " + this.templateName + ":" + error),
    };
    this.notificationsService.getEmailTemplates().subscribe(emailTemplatesObserver);
  }

  /**
   * Opens the modal prepopulated with the given content and listens for submit button clicks.
   * @param content Content to populate the form
   */
  public open(content: string): void {
    this.toggleSendEmailOpen()
    this.modalService.open(content, { ariaLabelledBy: "modal-title", centered: true, size: "lg" }).result.then(
      (result) => {
        if (result === "send" && this.customTextTag && this.customText) {
          // "Send" button clicked
          const sendEmailTemplatesObserver = {
            next: (response) => {
              this.emailSentEvent.emit(this.customText);
              this.toggleSendEmailOpen();
            },
            error: (error) => {
              console.error("Error when sending the email:", error)
              this.toggleSendEmailOpen();
            }
          };

          // Add our fields to the email parameters
          this.emailParameters["template_name"] = this.templateName;
          this.emailParameters[this.customTextTag] = this.customText;
          if (this.allowCC && this.ccAddresses) {
            this.emailParameters["cc_email"] = this.ccAddresses;
          }

          // Send the email and then emit edited data to parent component
          this.notificationsService.sendEmail(this.emailParameters).subscribe(sendEmailTemplatesObserver);
        } else if(result === "skip") {
          // User chose to skip sending an email
          this.emailSentEvent.emit(this.customText);
          this.toggleSendEmailOpen();
        } else {
          // Form was exited by clicking out of it or pressing ESC
          this.resetForm();
          this.toggleSendEmailOpen();
        }
      },
      (reason) => {
        // Form was exited using Cancel button or X
        this.resetForm();
        this.toggleSendEmailOpen();
      },
    );
  }

  /**
   * Toggles this modal being open and fires an event when the state changes.
   */
  private toggleSendEmailOpen(): void {
    this.isEmailModalOpen = !this.isEmailModalOpen;
    this.emailOpenEvent.emit(this.isEmailModalOpen);
  }

  /**
   * Resets the default text in the modal.
   */
  private resetForm(): void {
    this.customText = this.defaultText;
    this.ccAddresses = this.defaultCC;
  }

  /**
   * Defines how to update the email text from the form.
   * @param changes Changes to propagate from the form
   */
  public editText(changes: string): void {
    this.customText = changes;
  }

  /**
   * Defines how to update the CC email addresses from the form.
   * @param changes Changes to propagate from the form
   */
  public editCC(changes: string): void {
    this.ccAddresses = changes;
  }
}
